@@ -1409,7 +1409,7 @@ int establish_connection(struct perftest_comm *comm)
 		ptr = comm->rdma_params->servername ? &ethernet_client_connect : &ethernet_server_connect;
 
 		if ((*ptr)(comm)) {
-			fprintf(stderr,"Unable to open file descriptor for socket connection");
+			fprintf(stderr,"Unable to open file descriptor for socket connection\n");
 			return 1;
 		}
 	}
@@ -1810,13 +1810,6 @@ int ctx_close_connection(struct perftest_comm *comm,
 	}
 
 	if (!comm->rdma_params->use_rdma_cm && !comm->rdma_params->work_rdma_cm) {
-
-		if (write(comm->rdma_params->sockfd,"done",sizeof "done") != sizeof "done") {
-			perror(" Client write");
-			fprintf(stderr,"Couldn't write to socket\n");
-			return -1;
-		}
-
 		close(comm->rdma_params->sockfd);
 		return 0;
 	}
