@@ -123,7 +123,7 @@
 #define DEF_CACHE_LINE_SIZE (64)
 #define DEF_PAGE_SIZE (4096)
 #define DEF_FLOWS (1)
-#define RATE_VALUES_COUNT (18)
+#define RATE_VALUES_COUNT (22)
 #define DISABLED_CQ_MOD_VALUE    (1)
 #define MSG_SIZE_CQ_MOD_LIMIT (8192)
 
@@ -218,33 +218,33 @@
 /* Result print format */
 #define REPORT_FMT " %-7lu    %-10" PRIu64 "       %-7.2lf            %-7.2lf		   %-7.6lf"
 
-#define REPORT_FMT_JSON "MsgSize: %lu,\nn_iterations: %" PRIu64 ",\nBW_peak: %.2lf,\nBW_average: %.2lf,\nMsgRate: %.6lf,\n"
+#define REPORT_FMT_JSON "\"MsgSize\": %lu,\n\"n_iterations\": %" PRIu64 ",\n\"BW_peak\": %.2lf,\n\"BW_average\": %.2lf,\n\"MsgRate\": %.6lf"
 
 #define REPORT_FMT_EXT " %-7lu    %" PRIu64 "           %-7.6lf            %-7.6lf            %-7.6lf"
 
-#define REPORT_FMT_EXT_JSON "MsgSize: %lu,\nn_iterations: %" PRIu64 ",\nBW_peak: %.6lf,\nBW_average: %.6lf,\nMsgRate: %.6lf,\n"
+#define REPORT_FMT_EXT_JSON "\"MsgSize\": %lu,\n\"n_iterations\": %" PRIu64 ",\n\"BW_peak\": %.6lf,\n\"BW_average\": %.6lf,\n\"MsgRate\": %.6lf"
 
 #define REPORT_FMT_PER_PORT     " %-7lu    %-10" PRIu64 "     %-7.2lf            %-7.2lf		   %-7.6lf        %-7.2lf            %-7.6lf              %-7.2lf            %-7.6lf"
 
 #define REPORT_EXT	"\n"
-#define REPORT_EXT_JSON	""
+#define REPORT_EXT_JSON	"\n"
 
 #define REPORT_EXT_CPU_UTIL	"	    %-3.2f\n"
-#define REPORT_EXT_CPU_UTIL_JSON "CPU_util: %.2f,\n"
+#define REPORT_EXT_CPU_UTIL_JSON ",\n\"CPU_util\": %.2f\n"
 
 #define REPORT_FMT_QOS " %-7lu    %d           %lu           %-7.2lf            %-7.2lf                  %-7.6lf\n"
 
-#define REPORT_FMT_QOS_JSON "MsgSize: %lu,\nsl: %d,\nn_iterations: %lu,\nBW_peak: %.2lf,\nBW_average: %.2lf,\n MsgRate: %.6lf,\n"
+#define REPORT_FMT_QOS_JSON "\"MsgSize\": %lu,\nsl: %d,\n\"n_iterations\": %lu,\n\"BW_peak\": %.2lf,\n\"BW_average\": %.2lf,\n \"MsgRate\": %.6lf"
 
 /* Result print format for latency tests. */
 #define REPORT_FMT_LAT " %-7lu %" PRIu64 "          %-7.2f        %-7.2f      %-7.2f  	       %-7.2f     	%-7.2f		%-7.2f 		%-7.2f"
 
-#define REPORT_FMT_LAT_JSON "MsgSize: %lu,\nn_iterations: %" PRIu64 ",\nt_min: %.2f,\nt_max: %.2f,\nt_typical: %.2f,\nt_avg: %.2f,\n\
-t_stdev: %.2f,\npercentile_99: %.2f,\npercentile_99.9: %.2f,\n"
+#define REPORT_FMT_LAT_JSON "\"MsgSize\": %lu,\n\"n_iterations\": %" PRIu64 ",\n\"t_min\": %.2f,\n\"t_max\": %.2f,\n\"t_typical\": %.2f,\n\"t_avg\": %.2f,\n\
+\"t_stdev\": %.2f,\n\"percentile_99\": %.2f,\n\"percentile_99.9\": %.2f"
 
 #define REPORT_FMT_LAT_DUR " %-7lu       %" PRIu64 "            %-7.2f        %-7.2f"
 
-#define REPORT_FMT_LAT_DUR_JSON "MsgSize: %lu,\nn_iterations: %" PRIu64 ",\nt_avg: %.2f,\ntps_average: %.2f,\n"
+#define REPORT_FMT_LAT_DUR_JSON "\"MsgSize\": %lu,\n\"n_iterations\": %" PRIu64 ",\n\"t_avg\": %.2f,\n\"tps_average\": %.2f"
 
 #define REPORT_FMT_FS_RATE "%" PRIu64 "          %-7.2f        		%-7.2f      	%-7.2f  	       		%-7.2f     	%-7.2f"
 
@@ -323,7 +323,7 @@ t_stdev: %.2f,\npercentile_99: %.2f,\npercentile_99.9: %.2f,\n"
 } while (0)
 
 /* The Verb of the benchmark. */
-typedef enum { SEND , WRITE, READ, ATOMIC } VerbType;
+typedef enum { SEND , WRITE, WRITE_IMM, READ, ATOMIC } VerbType;
 
 /* The type of the test */
 typedef enum { LAT , BW , LAT_BY_BW, FS_RATE } TestType;
@@ -384,7 +384,7 @@ enum ctx_device {
 	ERDMA			= 29,
 	HNS			= 30,
 	CONNECTX8		= 31,
-  INTEL_GEN2		= 32,
+	INTEL_GEN2		= 32,
 };
 
 /* Units for rate limiter */
@@ -515,6 +515,7 @@ struct perftest_parameters {
 	int				duplex;
 	int				noPeak;
 	int				cq_mod;
+	int				fill_count;
 	int				req_cq_mod;
 	int 				spec;
 	int 				dualport;
@@ -569,6 +570,7 @@ struct perftest_parameters {
 	int				use_cuda_dmabuf;
 	int				rocm_device_id;
 	int				neuron_core_id;
+	int				use_neuron_dmabuf;
 	char				*hl_device_bus_id;
 	char				*mmap_file;
 	unsigned long			mmap_offset;
@@ -631,6 +633,7 @@ struct perftest_parameters {
 	char				*source_ip;
 	int 				has_source_ip;
 	int 			ah_allocated;
+	int				use_write_with_imm;
 };
 
 struct report_options {
@@ -666,8 +669,10 @@ static const struct rate_gbps_string RATE_VALUES[RATE_VALUES_COUNT] = {
 	{IBV_RATE_14_GBPS, "14"},
 	{IBV_RATE_20_GBPS, "20"},
 	{IBV_RATE_25_GBPS, "25"},
+	{IBV_RATE_28_GBPS, "28"},
 	{IBV_RATE_30_GBPS, "30"},
 	{IBV_RATE_40_GBPS, "40"},
+	{IBV_RATE_50_GBPS, "50"},
 	{IBV_RATE_56_GBPS, "56"},
 	{IBV_RATE_60_GBPS, "60"},
 	{IBV_RATE_80_GBPS, "80"},
@@ -677,6 +682,8 @@ static const struct rate_gbps_string RATE_VALUES[RATE_VALUES_COUNT] = {
 	{IBV_RATE_168_GBPS, "168"},
 	{IBV_RATE_200_GBPS, "200"},
 	{IBV_RATE_300_GBPS, "300"},
+	{IBV_RATE_400_GBPS, "400"},
+	{IBV_RATE_600_GBPS, "600"},
 	{IBV_RATE_MAX, "MAX"}
 };
 
